/*
   +----------------------------------------------------------------------+
   | This source file is subject to version 3.0 of the PHP license,       |
   | that is bundled with this package in the file LICENSE, and is        |
   | available through the world-wide-web at the following url:           |
   | http://www.php.net/license/3_0.txt.                                  |
   | If you did not receive a copy of the PHP license and are unable to   |
   | obtain it through the world-wide-web, please send a note to          |
   | license@php.net so we can mail you a copy immediately.               |
   +----------------------------------------------------------------------+
   | Authors: Hartmut Holzgraefe <hartmut@php.net>                        |
   +----------------------------------------------------------------------+
*/

/* $ Id: $ */ 

// {{{ CREATE and DROP statements for this UDF

/*
register the functions provided by this UDF module using
CREATE FUNCTION my_strcpy RETURNS STRING SONAME "my_strcpy.so";

unregister the functions provided by this UDF module using
DROP FUNCTION my_strcpy;
*/
// }}}

// {{{ standard header stuff
#ifdef STANDARD
#include <stdio.h>
#include <string.h>
#ifdef __WIN__
typedef unsigned __int64 ulonglong; /* Microsofts 64 bit types */
typedef __int64 longlong;
#else
typedef unsigned long long ulonglong;
typedef long long longlong;
#endif /*__WIN__*/
#else
#include <my_global.h>
#include <my_sys.h>
#endif
#include <mysql.h>
#include <m_ctype.h>
#include <m_string.h>       // To get strmov()

// }}}

#ifdef HAVE_DLOPEN

#include "udf_my_strcpy.h"

// {{{ prototypes

#ifdef  __cplusplus
extern "C" {
#endif
/* FUNCTION my_strcpy */
my_bool my_strcpy_init(UDF_INIT *initid, UDF_ARGS *args, char *message);
char * my_strcpy(UDF_INIT *initid, UDF_ARGS *args, char *result, unsigned long *length, char *is_null, char *error);
void my_strcpy_deinit(UDF_INIT *initid);

#ifdef  __cplusplus
}
#endif
// }}}

// {{{ UDF functions

// {{{ FUNCTION my_strcpy RETURNS STRING
struct my_strcpy_t {
  char * _resultbuf;
  unsigned long _resultbuf_len;
};

/* my_strcpy init function */
my_bool my_strcpy_init(UDF_INIT *initid, UDF_ARGS *args, char *message)
{
    initid->maybe_null = 0;
    initid->max_length = 255;

    if (args->arg_count != 1) {
        strcpy(message,"wrong number of parameters for my_strcpy()");
        return 1;
    }
    args->arg_type[0] = STRING_RESULT;

    return 0;
}

/* my_strcpy deinit function */
void my_strcpy_deinit(UDF_INIT *initid)
{
}

/* my_strcpy actual processing function */
char * my_strcpy(UDF_INIT *initid, UDF_ARGS *args, char *result, unsigned long *length, char *is_null, char *error)
{
    char *d=result, *s = (char *)args->args[0];

    *length = args->lengths[0];
    *is_null = 0;

    while (args->lengths[0]--) {
      *d++ = *s++;
    }
    *d = '\0';

    return result;
}

// }}}

// }}}

#else
#error your installation does not support loading UDFs
#endif /* HAVE_DLOPEN */

/*
 * Local variables:
 * tab-width: 4
 * c-basic-offset: 4
 * End:
 * vim600: noet sw=4 ts=4 fdm=marker
 * vim<600: noet sw=4 ts=4
 */
